@@ -39,16 +39,14 @@ GraphicsViewWindow::GraphicsViewWindow(MainWindow *pParent) :
 }
 
 void GraphicsViewWindow::updateTitle() {
-#ifdef Q_WS_MAEMO_5
-
-#else
     QString title = m_currentUrl.isEmpty() || !m_pGWebView ? "Macuco" : m_pGWebView->title();
+
+#ifndef Q_WS_MAEMO_5
     if ( m_webPage.isIphoneUserAgent() ) title += " - iPhone";
     title += QString(" - %1%").arg(QString::number(m_zoomFactor*100, 'f', 0));
-
-    setWindowTitle(title);
 #endif
 
+    setWindowTitle(title);
 }
 
 void GraphicsViewWindow::visitUrl(QUrl url) {
@@ -70,10 +68,3 @@ void GraphicsViewWindow::setZoom(qreal zoomFactor) {
     m_pGraphicsView->viewportItem()->resizeWebViewToFrame();
 #endif
 }
-
-MainWindow* GraphicsViewWindow::createWindow() {
-    if ( m_pParent ) return m_pParent->createWindow();
-
-    // top most
-    return new GraphicsViewWindow(this);
-}
