@@ -16,6 +16,7 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+#include "application.h"
 #include "mainwindow.h"
 
 #ifdef Q_WS_MAEMO_5
@@ -36,21 +37,20 @@
 #include <QPushButton>
 
 #include "fullscreenexitbutton.h"
+#include "waitingbutton.h"
 
 #include "gotodlg.h"
 #include "bookmark.h"
 #include "bookmarksdlg.h"
 #include "bookmarkdlg.h"
 
-#define SETTINGS_VERSION 1
-
 MainWindow::MainWindow(MainWindow *pParent) :
     QMainWindow(),
     m_webPage(this), m_pParent(pParent),
 #ifdef Q_WS_MAEMO_5
     m_pOrientationSelector(NULL), m_pUserAgentSelector(NULL),
 #endif
-    m_pBookmarks(NULL)
+    m_pBookmarks(NULL), m_pFullscreenAction(NULL), m_pWaitingButton(NULL)
 {
     setWindowTitle("Macuco");
 
@@ -93,13 +93,36 @@ MainWindow::MainWindow(MainWindow *pParent) :
     // connect(&m_webPage, SIGNAL(linkClicked(QUrl)), this, SLOT(linkClicked(QUrl)));
     connect(&m_webPage, SIGNAL(unsupportedContent(QNetworkReply*)), this, SLOT(unsupportedContent(QNetworkReply*)));
     connect(&m_webPage, SIGNAL(downloadRequested(QNetworkRequest)), this, SLOT(downloadRequested(QNetworkRequest)));
+
+#ifdef Q_WS_MAEMO_5
+    m_pWaitingButton = new WaitingButton(this);
+#endif
 }
 
 void MainWindow::startUp() {
+    Application &app(Application::Instance());
+
+    m_pBookmarks = &app.m_bookmarks;
+    m_zoomFactor = 1;
+    // setFullScreen(app.m_fullScreen);
+    setUserAgent( m_pParent ? m_pParent->m_webPage.isIphoneUserAgent() : app.m_fakeUserAgent );
+
+    if ( m_pParent && m_pParent->width()<m_pParent->height() ) {
+#ifdef Q_WS_MAEMO_5
+        // portrait
+        setAutoRotate(false);
+        setLandscape(false);
+#endif
+    } else {
+        // default orientation
+        setOrientation(app.m_orientation);
+    }
+
+    /*
     if ( m_pParent ) {
         // copy settings from mainwindow
         m_zoomFactor = m_pParent->m_zoomFactor;
-        m_pBookmarks = m_pParent->m_pBookmarks;
+        //m_pBookmarks = m_pParent->m_pBookmarks;
         setFullScreen(m_pParent->isFullScreen());
         setUserAgent( m_pParent->m_webPage.isIphoneUserAgent() );
 
@@ -110,12 +133,13 @@ void MainWindow::startUp() {
 #endif
 
     } else {
-        m_zoomFactor = 1;
+
         m_pBookmarks = new QList<Bookmark>();
 
         // load settings
         loadSettings();
     }
+    */
 
 #ifdef Q_WS_X11
 #ifndef Q_WS_MAEMO_5
@@ -125,33 +149,22 @@ void MainWindow::startUp() {
 #endif
 
     createMenu();
-
-    if ( !m_pParent ) on_actionHome_triggered();
-
     updateTitle();
 }
 
 MainWindow::~MainWindow() {
-    qDebug() << "~MainWindow";
-
-    if ( m_pParent ) {
-        qDebug() << ">> not top most";
-        m_pBookmarks = NULL;
-        return;
-    }
-
-    qDebug() << ">> top most window";
-    if ( m_pBookmarks ) {delete m_pBookmarks; m_pBookmarks = NULL;}
+    qDebug() << "~MainWindow, topMost: " << ( m_pParent==NULL );
 }
 
 void MainWindow::createMenu() {
     QMenu *pMenu = menuBar()->addMenu("&File");
-    pMenu->addAction("Home", this, SLOT(on_actionHome_triggered()));
+    pMenu->addAction("Home", this, SLOT(goHome()));
     pMenu->addAction("Go To...", this, SLOT(on_actionGo_To_triggered()));
+    pMenu->addAction("New Window", &Application::Instance(), SLOT(start()));
 
 #ifndef Q_WS_MAEMO_5
     pMenu->addSeparator();
-    pMenu->addAction("&Exit", this, SLOT(close()));
+    pMenu->addAction("&Exit", &Application::Instance(), SLOT(quit()));
 #endif
 
     pMenu = menuBar()->addMenu("&Bookmarks");
@@ -177,11 +190,16 @@ void MainWindow::createMenu() {
     m_pUserAgentSelector->setSelectedIndex(m_webPage.isIphoneUserAgent()?0:1);
     connect(m_pUserAgentSelector, SIGNAL(selectionChanged(int)), this, SLOT(userAgent_selectionChanged(int)));
     menuBar()->addAction(m_pUserAgentSelector);
+
+    if ( !QApplication::quitOnLastWindowClosed() ) {
+        pMenu->addAction("Shutdown", &Application::Instance(), SLOT(quit()));
+    }
 #else
     pMenu = menuBar()->addMenu("&View");
-    QAction *pFullscreen = pMenu->addAction("Fullscreen");
-    pFullscreen->setCheckable(true);
-    connect(pFullscreen, SIGNAL(toggled(bool)), this, SLOT(setFullScreen(bool)));
+    m_pFullscreenAction = pMenu->addAction("Fullscreen");
+    m_pFullscreenAction->setCheckable(true);
+    m_pFullscreenAction->setChecked(isFullScreen());
+    connect(m_pFullscreenAction, SIGNAL(toggled(bool)), this, SLOT(setFullScreen(bool)));
 
     pMenu->addSeparator();
 
@@ -199,6 +217,8 @@ void MainWindow::createMenu() {
     pMenu = menuBar()->addMenu("&About");
     pMenu->addAction("About Qt..", this, SLOT(aboutQt()));
 #endif
+
+    pMenu->addAction("About/FAQ/Donate", &Application::Instance(), SLOT(aboutMacuco()));
 }
 
 void MainWindow::aboutQt() {
@@ -208,23 +228,22 @@ void MainWindow::aboutQt() {
 void MainWindow::onUserAgentChanged(QAction *pAction) {
     qDebug() << pAction->text();
 
-    m_webPage.setIPhoneUserAgent(pAction->text().startsWith("iPhone"));
+    bool fakeUserAgent = pAction->text().startsWith("iPhone");
+    m_webPage.setIPhoneUserAgent(fakeUserAgent);
     updateTitle();
+
+    // change default settings
+    Application::Instance().m_fakeUserAgent = fakeUserAgent;
 }
 
 void MainWindow::userAgent_selectionChanged(int index) {
-    switch ( index ) {
-        case 0: // iphone
-            m_webPage.setIPhoneUserAgent();
-            break;
+    m_webPage.setIPhoneUserAgent(index==0);
 
-        case 1: // default
-            m_webPage.setIPhoneUserAgent(false);
-            break;
-    }
+    // change default settings
+    Application::Instance().m_fakeUserAgent = index==0;
 }
 
-void MainWindow::orientation_selectionChanged(int index) {
+void MainWindow::setOrientation(int index) {
 #ifdef Q_WS_MAEMO_5
     switch ( index ) {
         case 0: // auto
@@ -244,60 +263,18 @@ void MainWindow::orientation_selectionChanged(int index) {
 #endif
 }
 
+void MainWindow::orientation_selectionChanged(int index) {
+    // update default orientation
+    Application::Instance().m_orientation = index;
+
+    setOrientation(index);
+}
+
 MainWindow* MainWindow::topMost() {
     if ( m_pParent ) return m_pParent->topMost();
     return this;
 }
 
-void MainWindow::loadSettings() {
-    qDebug() << "load settings";
-
-    Bookmark::registerMeataType();
-
-    QSettings settings("Macuco", "Macuco");
-    int version = settings.value("version", 0).toInt();
-    if ( version<SETTINGS_VERSION ) {
-        qDebug() << "previous version of settings file";
-        //...
-    }
-
-    Q_ASSERT(m_pBookmarks);
-    (*m_pBookmarks) = settings.value("bookmarks").value< QList<Bookmark> >();
-    setFullScreen(settings.value("fullScreen", false).toBool());
-    setUserAgent(settings.value("iphoneUserAgent", true).toBool());
-
-#ifdef Q_WS_MAEMO_5
-    int orientation = settings.value("orientation", 0).toInt();
-    orientation_selectionChanged(orientation);
-#endif
-
-    qDebug() << "try to get icons";
-    // get page icons
-    // (work around for not getting the icons the first time - have to investigate)
-    for ( int c=0; c<m_pBookmarks->size(); c++ ) {
-        QWebSettings::iconForUrl((*m_pBookmarks)[c].url());
-    }
-
-    qDebug() << ">> done";
-}
-
-void MainWindow::saveSettings() {
-    QSettings settings("Macuco", "Macuco");
-    settings.setValue("version", SETTINGS_VERSION);
-    settings.setValue("fullScreen", isFullScreen());
-    settings.setValue("iphoneUserAgent", m_webPage.isIphoneUserAgent());
-
-#ifdef Q_WS_MAEMO_5
-    Q_ASSERT(m_pOrientationSelector);
-    settings.setValue("orientation", m_pOrientationSelector->selectedIndex());
-#endif
-
-    Q_ASSERT(m_pBookmarks);
-    QVariant bookmarks;
-    bookmarks.setValue(*m_pBookmarks);
-    settings.setValue("bookmarks", bookmarks);
-}
-
 void MainWindow::showLoading() {
     m_errors.clear();
 
@@ -305,14 +282,43 @@ void MainWindow::showLoading() {
     // http://qt.nokia.com/doc/qt-maemo-4.6/maemo5-windowstates.html
     setAttribute(Qt::WA_Maemo5ShowProgressIndicator, true);
 #else
-    setCursor(QCursor(Qt::WaitCursor));
-    /*
+    QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
+        /*
     m_pErrorBox->hide();
     hideOverlayOnPage();
 
     m_pLoadingLabel->show();
     */
 #endif
+
+    if ( m_pWaitingButton ) m_pWaitingButton->showAndStart();
+}
+
+void MainWindow::hideLoading(bool success) {
+    if ( m_pWaitingButton ) m_pWaitingButton->stopAndHide();
+
+    setViewport();
+
+#ifdef Q_WS_MAEMO_5
+    setAttribute(Qt::WA_Maemo5ShowProgressIndicator, false);
+
+    // use notification
+    // ...
+
+    if ( !success ) {
+        QString msg("<b>Loading Error</b><br/>");
+        if ( m_errors.length()>0 ) msg+=m_errors.join("<br/>"); else msg+=m_currentUrl.toString();
+        QMaemo5InformationBox::information(this,
+            msg,
+            QMaemo5InformationBox::DefaultTimeout
+        );
+        return;
+    }
+#else
+    QApplication::restoreOverrideCursor();
+
+    if ( !success ) QMessageBox::warning(this, "Loading Error", m_errors.join("\n"));
+#endif
 }
 
 void MainWindow::changeUrl(QUrl url) {
@@ -351,13 +357,32 @@ void MainWindow::zoomFit() {
     }
 }
 
-void MainWindow::resizeEvent(QResizeEvent* event) {
-    QWidget::resizeEvent(event);
+void MainWindow::changeEvent(QEvent *pEvent) {
+    QMainWindow::changeEvent(pEvent);
+
+    Q_ASSERT(pEvent);
+
+    if ( pEvent->type()== QEvent::WindowStateChange ) {
+        if ( !m_pFullscreenAction ) return;
+        m_pFullscreenAction->setChecked(isFullScreen());
+    }
+
 
-    setViewport();
 }
 
-void MainWindow::closeEvent(QCloseEvent *event)
+void MainWindow::resizeEvent(QResizeEvent* pEvent) {
+    QWidget::resizeEvent(pEvent);
+
+    setViewport();
+
+    if ( m_pWaitingButton ) {
+        // center button
+        m_pWaitingButton->move(0,height()-m_pWaitingButton->height());
+        // m_pWaitingButton->move((width()-m_pWaitingButton->width())/2,(height()-m_pWaitingButton->height())/2);
+    }
+}
+
+void MainWindow::closeEvent(QCloseEvent *pEvent)
 {
 #ifdef Q_WS_MAEMO_5
     grabZoomKeys(false);
@@ -365,11 +390,11 @@ void MainWindow::closeEvent(QCloseEvent *event)
 
     if ( !m_pParent ) {
         // top most
-        saveSettings();
+        // saveSettings();
         m_webPage.saveCookies();
     }
 
-    event->accept();
+    pEvent->accept();
 }
 
 void MainWindow::zoomIn() {
@@ -417,7 +442,7 @@ void MainWindow::keyPressEvent(QKeyEvent *event)
 
         case Qt::Key_H:
             if ( (event->modifiers()&Qt::ControlModifier) != Qt::ControlModifier  ) break;
-            on_actionHome_triggered();
+            goHome();
             break;
 
         case Qt::Key_F:
@@ -441,7 +466,7 @@ void MainWindow::enableWebkitAutoSize() {
     m_webPage.mainFrame()->evaluateJavaScript("document.body.style.WebkitTextSizeAdjust = 'auto'");
 }
 
-void MainWindow::on_actionHome_triggered() {
+void MainWindow::goHome() {
     visitUrl("http://www.crochik.com/macuco");
 }
 
@@ -483,6 +508,7 @@ bool MainWindow::isFullScreen() {
     return windowState() & Qt::WindowFullScreen;
 }
 
+
 void MainWindow::on_actionGo_To_triggered()
 {
     GoToDlg dlg(this);
@@ -600,11 +626,6 @@ void MainWindow::grabZoomKeys(bool grab)
 }
 #endif
 
-
-void MainWindow::on_actionUser_Agent_triggered() {
-    setUserAgent(!m_webPage.isIphoneUserAgent());
-}
-
 void MainWindow::setUserAgent(bool iphone/*=true*/) {
     m_webPage.setIPhoneUserAgent(iphone);
 
@@ -707,27 +728,4 @@ bool MainWindow::setViewport() {
     return false;
 }
 
-void MainWindow::hideLoading(bool success) {
-    setViewport();
 
-#ifdef Q_WS_MAEMO_5
-    setAttribute(Qt::WA_Maemo5ShowProgressIndicator, false);
-
-    // use notification
-    // ...
-
-    if ( !success ) {
-        QString msg("<b>Loading Error</b><br/>");
-        if ( m_errors.length()>0 ) msg+=m_errors.join("<br/>"); else msg+=m_currentUrl.toString();
-        QMaemo5InformationBox::information(this,
-            msg,
-            QMaemo5InformationBox::DefaultTimeout
-        );
-        return;
-    }
-#else
-    setCursor(QCursor(Qt::ArrowCursor));
-
-    if ( !success ) QMessageBox::warning(this, "Loading Error", m_errors.join("\n"));
-#endif
-}
