@@ -33,6 +33,7 @@
 #include "bookmark.h"
 #include "webpage.h"
 
+class WaitingButton;
 
 namespace Ui {
     class MainWindow;
@@ -54,9 +55,6 @@ public:
     void visitUrl(QString);
     void setUserAgent(bool iphone=true);
 
-public:
-    virtual MainWindow *createWindow() =0;
-
 protected:
     virtual void updateTitle()=0;
     virtual void visitUrl(QUrl)=0;
@@ -64,23 +62,12 @@ protected:
 
 protected:
     void startUp();
-    void loadSettings();
-    void saveSettings();
-
     bool isFullScreen();
-
-    // reimplement resize event to adapt layout
-    void resizeEvent(QResizeEvent* event);
-
-    //intercepts close event
-    void closeEvent(QCloseEvent *event);
-
     void zoomIn();
     void zoomOut();
     void zoomFit();
     void createMenu();
     bool setViewport();
-
     void enableWebkitAutoSize();
 
 #ifdef Q_WS_MAEMO_5
@@ -91,19 +78,26 @@ protected:
     void setAutoRotate(bool autoRotate=true);
 #endif
 
+    void setOrientation(int index);
+
+protected: //events
+    void resizeEvent(QResizeEvent* event);
+    void closeEvent(QCloseEvent *event);
+    void changeEvent(QEvent *event);
+
+public slots:
+    void goHome();
+
 protected slots:
-    void on_actionUser_Agent_triggered();
     void on_actionAdd_Bookmark_triggered();
     void on_actionGo_To_triggered();
     void on_actionAuto_Rotate_triggered();
     void on_actionLandscape_triggered();
     void on_actionBookmarks_triggered();
-    void on_actionHome_triggered();
 
     void onUserAgentChanged(QAction *pAction);
 
     void aboutQt();
-    // void switchView();
     void setFullScreen(bool full=true);
     void showLoading(); //called when a "loading page" is started
     void changeUrl(QUrl url); //called when the url is changed. The url will be used on the retry action
@@ -135,6 +129,9 @@ protected:
     WebPage         m_webPage;
     MainWindow      *m_pParent;
 
+    QAction         *m_pFullscreenAction;
+    WaitingButton   *m_pWaitingButton;
+
 #ifdef Q_WS_MAEMO_5
     PickSelectorAction  *m_pUserAgentSelector;
     PickSelectorAction  *m_pOrientationSelector;
